import { ReflectiveInjector } from "@angular/core";
import { getMetadataStorage } from "../odoo/utils/metadata-storage";

/**
 *
 * ██████╗  █████╗ ███████╗███████╗██╗   ██╗██╗███████╗██╗    ██╗
 * ██╔══██╗██╔══██╗██╔════╝██╔════╝██║   ██║██║██╔════╝██║    ██║
 * ██████╔╝███████║███████╗█████╗  ██║   ██║██║█████╗  ██║ █╗ ██║
 * ██╔══██╗██╔══██║╚════██║██╔══╝  ╚██╗ ██╔╝██║██╔══╝  ██║███╗██║
 * ██████╔╝██║  ██║███████║███████╗ ╚████╔╝ ██║███████╗╚███╔███╔╝
 * ╚═════╝ ╚═╝  ╚═╝╚══════╝╚══════╝  ╚═══╝  ╚═╝╚══════╝ ╚══╝╚══╝
 *
 * Clase de la cual hereda tanto las clases que manejan vistas de lista como la de detalles
 * y exponen métodos comunes entre ambos.
 *
 */
export abstract class BaseView<T> {

    type: T = {} as T;
    model: any = {};
    fields: Array<any>;
    injector: any = {};
    title: string;

    constructor(c: { new (): T; }, ...injectables: any[]) {
        this.type = new c();
        this.model = getMetadataStorage().models.filterByTarget(this.type.constructor).items.shift();
        this.fields = getMetadataStorage().fields.filterByTarget(this.type.constructor).items;
        this.injector = ReflectiveInjector.resolveAndCreate(injectables);
        this.title = "Sin título";
    }

    // ------------------------------------------------------------
    //   ██████╗ ███████╗████████╗████████╗███████╗██████╗ ███████╗
    //  ██╔════╝ ██╔════╝╚══██╔══╝╚══██╔══╝██╔════╝██╔══██╗██╔════╝
    //  ██║  ███╗█████╗     ██║      ██║   █████╗  ██████╔╝███████╗
    //  ██║   ██║██╔══╝     ██║      ██║   ██╔══╝  ██╔══██╗╚════██║
    //  ╚██████╔╝███████╗   ██║      ██║   ███████╗██║  ██║███████║
    //   ╚═════╝ ╚══════╝   ╚═╝      ╚═╝   ╚══════╝╚═╝  ╚═╝╚══════╝
    // ------------------------------------------------------------

    /**
     * 
     * @param injectable 
     */
    getInjectable(injectable: any): any {
        try {
            return this.injector.get(injectable);
        } catch (error) {
            return undefined;
        }
    }

    /**
     * 
     */
    getTitle(): string {
        return this.title;
    }

       /**
     * 
     */
    getType(): T {
        return this.type;
    }

    /**
     * 
     */
    getModel(): any {
        return this.model;
    }

    /**
     * 
     */
    getModelName(): string {
        return this.getModel().name;
    }

    /**
     * 
     */
    getFields(): Array<any> {
        return this.fields;
    }

    /**
     * 
     */
    getFieldsRelated(): Array<any> {
        return this.getFields().filter(field => {
            return !!field.options.related;
        });
    }

    /**
     * 
     */
    getInjector(): any {
        return this.injector;
    }

    // -----------------------------------------------------------
    //  ███████╗███████╗████████╗████████╗███████╗██████╗ ███████╗
    //  ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██╔════╝██╔══██╗██╔════╝
    //  ███████╗█████╗     ██║      ██║   █████╗  ██████╔╝███████╗
    //  ╚════██║██╔══╝     ██║      ██║   ██╔══╝  ██╔══██╗╚════██║
    //  ███████║███████╗   ██║      ██║   ███████╗██║  ██║███████║
    //  ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚══════╝╚═╝  ╚═╝╚══════╝
    // -----------------------------------------------------------
    
    /**
     * 
     * @param title 
     */
    setTitle(title: string): void {
        this.title = title;
    }
}